-- hds header_start
--
-- VHDL Entity CERC_be_fed.i2c_ttc_engine.symbol
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 19:35:05 12/04/05
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

entity i2c_ttc_engine is
   port( 
      ce_i2c         : in     std_logic;
      ce_i2c_rcv     : in     std_logic;
      data_to_ttc    : in     std_logic_vector (7 downto 0);
      i2c_abort      : in     std_logic;
      i2c_init       : in     std_logic;
      i2c_rd_nwr     : in     std_logic;
      i2c_rst        : in     std_logic;
      iclk40         : in     std_logic;
      scl_source     : in     std_logic;
      sda_in         : in     std_logic;
      sr_out         : in     std_logic;
      ttc_pointer    : in     std_logic_vector (4 downto 0);
      dir_sda        : out    std_logic;
      en_sr_in       : out    std_logic;
      en_sr_out      : out    std_logic;
      i2c_busy       : out    std_logic;
      i2c_err        : out    std_logic;
      ld_sr_out      : out    std_logic;
      scl_en         : out    std_logic;
      sda_out        : out    std_logic;
      sel_pointer    : out    std_logic;
      set_vme_status : out    std_logic
   );

-- Declarations

end i2c_ttc_engine ;


-- hds interface_end

--
-- VHDL Architecture CERC_be_fed.i2c_ttc_engine.fsm
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 19:35:05 12/04/05
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

architecture fsm of i2c_ttc_engine is

   -- Architecture Declarations
   signal icount: natural range 0 to 8;
   signal pass_two: std_logic;
   signal abort: std_logic;

   type state_type is (
      init,
      ld_sr_pointer,
      done,
      ld_sr_data,
      i2c_start,
      snd_addr,
      rcv_ack1,
      snd_data,
      rcv_ack2,
      i2c_stop1,
      rcv_data,
      snd_nack,
      error,
      sr_out1,
      sr_out2,
      sr_in,
      start_clk,
      i2c_stop_scl,
      i2c_stop2
   );

   -- State vector declaration
   attribute state_vector : string;
   attribute state_vector of fsm : architecture is "current_state" ;


   -- Declare current and next state signals
   signal current_state : state_type ;
   signal next_state : state_type ;

   -- Declare any pre-registered internal signals
   signal dir_sda_int : std_logic  ;
   signal en_sr_in_int : std_logic  ;
   signal en_sr_out_int : std_logic  ;
   signal i2c_busy_int : std_logic  ;
   signal i2c_err_int : std_logic  ;
   signal ld_sr_out_int : std_logic  ;
   signal scl_en_int : std_logic  ;
   signal sda_out_int : std_logic  ;
   signal set_vme_status_int : std_logic  ;

begin

   ----------------------------------------------------------------------------
   clocked : process(
      iclk40,
      i2c_rst
   )
   ----------------------------------------------------------------------------
   begin
      if (i2c_rst = '1') then
         current_state <= init;
         -- Reset Values
         dir_sda <= '1';
         en_sr_in <= '0';
         en_sr_out <= '0';
         i2c_busy <= '0';
         i2c_err <= '0';
         ld_sr_out <= '0';
         scl_en <= '0';
         sda_out <= '1';
         set_vme_status <= '0';
         abort <= '0';
         icount <= 0;
         pass_two <= '0';
      elsif (iclk40'event and iclk40 = '1') then
         current_state <= next_state;
         -- Registered output assignments
         dir_sda <= dir_sda_int;
         en_sr_in <= en_sr_in_int;
         en_sr_out <= en_sr_out_int;
         i2c_busy <= i2c_busy_int;
         i2c_err <= i2c_err_int;
         ld_sr_out <= ld_sr_out_int;
         scl_en <= scl_en_int;
         sda_out <= sda_out_int;
         set_vme_status <= set_vme_status_int;

         -- Default Assignment To Internals
         abort <= i2c_abort or abort;
         icount <= 0;
         pass_two <= pass_two;

         -- Combined Actions for internal signals only
         case current_state is
         when init =>
            pass_two <= '0';
            abort <= '0';
         when ld_sr_data =>
            pass_two <= '1';
         when i2c_start =>
            icount <= 0;
         when snd_addr =>
            icount <= icount;
         when rcv_ack1 =>
            icount <= 0;
         when snd_data =>
            icount <= icount;
         when rcv_data =>
            icount <= icount;
         when error =>
            abort <= '1';
         when sr_out1 =>
            icount <= icount + 1;
         when sr_out2 =>
            icount <= icount + 1;
         when sr_in =>
            icount <= icount + 1;
         when others =>
            null;
         end case;

      end if;

   end process clocked;

   ----------------------------------------------------------------------------
   nextstate : process (
      abort,
      ce_i2c,
      ce_i2c_rcv,
      current_state,
      i2c_init,
      i2c_rd_nwr,
      icount,
      pass_two,
      scl_source,
      sda_in
   )
   ----------------------------------------------------------------------------
   begin
      case current_state is
      when init =>
         if (i2c_init = '1') then
            next_state <= ld_sr_pointer;
         else
            next_state <= init;
         end if;
      when ld_sr_pointer =>
         if (ce_i2c = '1') then
            next_state <= i2c_start;
         else
            next_state <= ld_sr_pointer;
         end if;
      when done =>
            next_state <= init;
      when ld_sr_data =>
         if (ce_i2c = '1') then
            next_state <= i2c_start;
         else
            next_state <= ld_sr_data;
         end if;
      when i2c_start =>
         if (scl_source = '1') then
            next_state <= start_clk;
         else
            next_state <= i2c_start;
         end if;
      when snd_addr =>
         if (ce_i2c = '1') then
            next_state <= sr_out1;
         else
            next_state <= snd_addr;
         end if;
      when rcv_ack1 =>
         if ((ce_i2c_rcv = '1') and 
         (sda_in /= '0')) then
            next_state <= error;
         elsif ((ce_i2c = '1') and 
         (i2c_rd_nwr = '1') and
         (pass_two ='1')) then
            next_state <= rcv_data;
         elsif ((ce_i2c = '1')) then
            next_state <= snd_data;
         else
            next_state <= rcv_ack1;
         end if;
      when snd_data =>
         if ((ce_i2c = '1') and 
         (icount = 7)) then
            next_state <= rcv_ack2;
         elsif (ce_i2c = '1') then
            next_state <= sr_out2;
         else
            next_state <= snd_data;
         end if;
      when rcv_ack2 =>
         if ((ce_i2c_rcv = '1') and 
         (sda_in /= '0')) then
            next_state <= error;
         elsif (ce_i2c = '1') then
            next_state <= i2c_stop1;
         else
            next_state <= rcv_ack2;
         end if;
      when i2c_stop1 =>
         if (scl_source = '1') then
            next_state <= i2c_stop_scl;
         else
            next_state <= i2c_stop1;
         end if;
      when rcv_data =>
         if ((ce_i2c = '1') and 
         (icount = 8)) then
            next_state <= snd_nack;
         elsif (ce_i2c_rcv = '1') then
            next_state <= sr_in;
         else
            next_state <= rcv_data;
         end if;
      when snd_nack =>
         if (ce_i2c = '1') then
            next_state <= i2c_stop1;
         else
            next_state <= snd_nack;
         end if;
      when error =>
         if (ce_i2c = '1') then
            next_state <= i2c_stop1;
         else
            next_state <= error;
         end if;
      when sr_out1 =>
         if (icount = 7) then
            next_state <= rcv_ack1;
         else
            next_state <= snd_addr;
         end if;
      when sr_out2 =>
            next_state <= snd_data;
      when sr_in =>
            next_state <= rcv_data;
      when start_clk =>
         if (ce_i2c = '1') then
            next_state <= snd_addr;
         else
            next_state <= start_clk;
         end if;
      when i2c_stop_scl =>
         if (ce_i2c = '1') then
            next_state <= i2c_stop2;
         else
            next_state <= i2c_stop_scl;
         end if;
      when i2c_stop2 =>
         if ((ce_i2c = '1') and ((pass_two = '1') or 
         (abort = '1'))) then
            next_state <= done;
         elsif (ce_i2c = '1') then
            next_state <= ld_sr_data;
         else
            next_state <= i2c_stop2;
         end if;
      when others =>
         next_state <= init;
      end case;

   end process nextstate;

   ----------------------------------------------------------------------------
   output : process (
      current_state,
      sr_out
   )
   ----------------------------------------------------------------------------
   begin
      -- Default Assignment
      dir_sda_int <= '0';
      en_sr_in_int <= '0';
      en_sr_out_int <= '0';
      i2c_busy_int <= '1';
      i2c_err_int <= '0';
      ld_sr_out_int <= '0';
      scl_en_int <= '0';
      sda_out_int <= '1';
      set_vme_status_int <= '0';
      -- Default Assignment To Internals

      -- Combined Actions
      case current_state is
      when init =>
         i2c_busy_int <= '0';
         dir_sda_int <= '1';
         i2c_err_int <= '0';
      when ld_sr_pointer =>
         ld_sr_out_int <= '1';
      when done =>
         set_vme_status_int <= '1';
      when ld_sr_data =>
         ld_sr_out_int <= '1';
      when i2c_start =>
         dir_sda_int <= '0';
         sda_out_int <= '0';
      when snd_addr =>
         dir_sda_int <= '0';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      when rcv_ack1 =>
         dir_sda_int <= '1';
         scl_en_int <= '1';
         sda_out_int <= sr_out;
      when snd_data =>
         dir_sda_int <= '0';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      when rcv_ack2 =>
         dir_sda_int <= '1';
         scl_en_int <= '1';
         sda_out_int <='0';
      when i2c_stop1 =>
         sda_out_int <= '0';
         scl_en_int <= '1';
         -- i2c_err_int <= i2c_err_int;
      when rcv_data =>
         dir_sda_int <= '1';
         scl_en_int <= '1';
      when snd_nack =>
         dir_sda_int <= '0';
         sda_out_int <= '1';
         scl_en_int <= '1';
      when error =>
         dir_sda_int <= '1';
         i2c_err_int <= '1';
         scl_en_int <= '1';
         sda_out_int <='0';
      when sr_out1 =>
         dir_sda_int <= '0';
         en_sr_out_int <= '1';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      when sr_out2 =>
         dir_sda_int <= '0';
         en_sr_out_int <= '1';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      when sr_in =>
         dir_sda_int <= '1';
         en_sr_in_int <= '1';
         scl_en_int <= '1';
      when start_clk =>
         dir_sda_int <= '0';
         sda_out_int <= '0';
         scl_en_int <= '1';
      when i2c_stop_scl =>
         sda_out_int <= '0';
         -- i2c_err_int <= i2c_err_int;
      when i2c_stop2 =>
         sda_out_int <= '1';
         -- i2c_err_int <= i2c_err_int;
      when others =>
         null;
      end case;

   end process output;

   -- Concurrent Statements
sel_pointer <= not pass_two;

end fsm;
