-- hds header_start

-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity burst_sys is
   port( 
      rst                : in     std_logic;
      clk                : in     std_logic;
      db                 : inout  std_logic_vector (31 downto 0);
      db_rd              : in     std_logic;
      db_wr              : in     std_logic;
      ren_burst_i        : in     std_logic;
      ren_control_i      : in     std_logic;
      ren_busy_trig_to_i : in     std_logic;
      ren_burst_to_i     : in     std_logic;
      tbto_en_i          : in     std_logic;
      burst_en_i         : in     std_logic;
      bto_en_i           : in     std_logic;
      ibto_en_i          : in     std_logic;
      --bburst_en_i     : in     std_logic;
      busy_trig_i        : in     std_logic;
      trig_sync_i        : in     std_logic;
      oa_clr_busy_trig_o : out    std_logic;
      oa_busy_force_o    : out    std_logic
   );

-- Declarations

end burst_sys ;


-- hds interface_end
architecture rtl of burst_sys is


	-- Trigger Busy Timer
	signal tbto_timer 		: std_logic_vector(31 downto 0);
	signal tbto_timer_en 	: std_logic;
	signal tbto_timer_done  : std_logic;
	

	-- Long Timer
	signal long_timer_val : std_logic_vector(15 downto 0);
	signal long_timer 		: std_logic_vector(15 downto 0);
	signal long_timer_en 	: std_logic;
	signal long_timer_done  : std_logic;
	signal count_msec 		: std_logic_vector(15 downto 0);
	signal count_msec_ro		: std_logic;
		
	-- Burst Counter
	signal bcount 				: std_logic_vector(15 downto 0);
	signal bcount_en 			: std_logic;
	signal bcount_done    	: std_logic;
		
	-- OA Clear Trig Busy Siganls
	signal clr_busy_trig_reg : 	std_logic;
	signal clr_busy_trig_to : 	std_logic;
	signal clr_busy_trig_burst : 	std_logic;
	
	-- Write zero signals
	signal clr_busy_burst 	: 	std_logic;
	signal clr_busy_bto 		: 	std_logic;
	signal clr_busy_ibto		: 	std_logic;
--	signal clr_busy_bburst 	: 	std_logic;
	signal clr_busy_to		:	std_logic;
	
	signal busy_force_reg 	: 	std_logic;
		
	signal busy_burst 		: 	std_logic;
	signal busy_bto 			: 	std_logic;
	signal busy_ibto 			: 	std_logic;
--	signal busy_bburst 		: 	std_logic;
	
	--Registers
	signal reg_busy_trig_to	: std_logic_vector(31 downto 0);
	signal reg_control_rd 	: std_logic_vector(31 downto 0);
	signal reg_control_wr 	: std_logic_vector(31 downto 0);
	signal reg_burst			: std_logic_vector(31 downto 0);
	signal reg_burst_to		: std_logic_vector(31 downto 0);

------------------------------------------------------------------------------
begin
	
	oa_busy_force_o <= busy_force_reg or
							 busy_burst or
							 busy_bto or
							 busy_ibto;
							 
	oa_clr_busy_trig_o 	 <= clr_busy_trig_reg or 
									 clr_busy_trig_burst or 
									 clr_busy_trig_to;


	---------------------------------------------------------------------------
	-- Trigger Busy Timeout
	---------------------------------------------------------------------------
	
	-- Register
	reg_busy_trig_to_proc : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				reg_busy_trig_to	<= (others => '0');
																
			elsif (ren_busy_trig_to_i and db_wr) = '1' then
				reg_busy_trig_to <= db;
				
			end if;
		end if;
	end process;
	
	db <= reg_busy_trig_to when (ren_busy_trig_to_i and db_rd) = '1' else (others => 'Z');
	
	
	-- Timer/Controller
		
	busy_timeout_proc : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				clr_busy_trig_to <= '0';
				tbto_timer_en <= '0';
				
			else
				-- default
				clr_busy_trig_to <= '0';
				
				if (tbto_en_i = '1') then
					if (trig_sync_i = '1') then
						tbto_timer_en 	<= '1';
						
					
					elsif (tbto_timer_done = '1') then
						tbto_timer_en 	<= '0';
						clr_busy_trig_to    <= '1';
						
					end if;
				end if;
			end if;
		end if;
	end process;
			
	
	tbto_timer_proc : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') or (tbto_timer_en = '0') then
				tbto_timer	<= (others => '0');
				tbto_timer_done <= '0';
												
			else
				if (tbto_timer = reg_busy_trig_to) then
					tbto_timer_done <= '1';
				
				else
					tbto_timer_done <= '0';
					tbto_timer <= tbto_timer + '1';
					
				end if;		
			end if;
		end if;
	end process;

	---------------------------------------------------------------------------
	
	-- Reg Burst
	reg_burst_proc : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				reg_burst	<= (others => '0');
																
			elsif (ren_burst_i and db_wr) = '1' then
				reg_burst <= db;
				
			end if;
		end if;
	end process;
	
	db <= reg_burst when (ren_burst_i and db_rd) = '1' else (others => 'Z');
	
	
	
	-- Reg Burst Timeouts
	reg_burst_to_proc : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				reg_burst_to	<= (others => '0');
																
			elsif (ren_burst_to_i and db_wr) = '1' then
				reg_burst_to <= db;
				
			end if;
		end if;
	end process;
	
	db <= reg_burst_to when (ren_burst_to_i and db_rd) = '1' else (others => 'Z');
	
	
	---------------------------------------------------------------------------
	-- The Burst Machine
	---------------------------------------------------------------------------
	burst_machine : block
		type state_type is (	RESET, 
									IDLE, 
									BURST_START, 
									WAIT_BURST_END, 
									IBTO_START, 
									WAIT_IBTO_END 
									);
	
		signal state 		: state_type;
		signal next_state : state_type;

	---------------------------------------------------------------------------
	begin

		next_state_and_output : process (
			state,
			burst_en_i, bto_en_i, ibto_en_i, 
			clr_busy_burst, clr_busy_bto, clr_busy_ibto,
			bcount_done, long_timer_done
			)
		----------------------------------------------
		begin
		
			-- defaults
			clr_busy_trig_burst <= '0';
		
	
			case state is
				------------------------------------------------------------
				when RESET =>
					long_timer_en 	<= '0';
					bcount_en		<= '0';
										
					busy_burst 		<= '0';
					busy_bto 		<= '0';
					busy_ibto 		<= '0';
					
					clr_busy_trig_burst <= '0';
																			
					next_state <= IDLE;
			
				------------------------------------------------------------
				when IDLE =>
					next_state <= IDLE;
				
					busy_burst 		<= '1';
					bcount_en		<= '0';
					long_timer_en	<= '0';
						
					if (clr_busy_burst = '1') then
						next_state <= BURST_START;
							
					end if;
									
				------------------------------------------------------------	
				when BURST_START =>
								
					busy_burst 	<= '0';
					busy_bto 	<= '0';
					bcount_en	<= '1';
					clr_busy_trig_burst <= '1';
					
											
					if (bto_en_i = '1') then 
						long_timer_val <= reg_burst_to(15 downto 0);
						long_timer_en	<= '1';
								
					end if;
					
					next_state <= WAIT_BURST_END;										
					
				
				------------------------------------------------------------
				when WAIT_BURST_END =>
					next_state <= WAIT_BURST_END;
					
					if (bcount_done = '1') or (long_timer_done = '1') then
						bcount_en		<= '0';
						busy_burst 		<= '1';
						long_timer_en	<= '0';
											
						
						if (bto_en_i = '1') and (long_timer_done = '1') then 
							busy_bto 		<= '1';
															
						end if;
						
						if (ibto_en_i = '1') then
							next_state <= IBTO_START;
						
						else
							next_state <= IDLE;
						
						end if;
									
					else
						if (bto_en_i = '0') or (clr_busy_bto = '1') then	-- in case the bto gets disabled mid burst...
							busy_bto 		<= '0';
							long_timer_en  <= '0';
							
						end if;
															
					end if;
					
				------------------------------------------------------------
				when IBTO_START =>
									
					long_timer_val <= reg_burst_to(31 downto 16);
					long_timer_en	<= '1';
					busy_ibto		<= '1';
												
					next_state <= WAIT_IBTO_END;
							
				------------------------------------------------------------
				when WAIT_IBTO_END =>
					next_state <= WAIT_IBTO_END;
					
					if (ibto_en_i = '0') or (clr_busy_ibto = '1') then -- incase it is cancelled midway
						long_timer_en	<= '0';
						busy_ibto		<= '0';
						next_state 		<= IDLE;
						
					elsif (long_timer_done = '1') then
							busy_ibto 		<= '0';
							long_timer_en	<= '0';
							next_state 		<= BURST_START;
						
					end if;
					
					if (bto_en_i = '0') or (clr_busy_bto = '1') then	-- in case the bto gets disabled mid burst...
						busy_bto 		<= '0';
														
					end if;
										
				
				------------------------------------------------------------
				when others => null;
				
			end case;	
			
		------------------------------------------------------------------------
		end process;
		
		
		
		
			
		------------------------------
		state_reg : process (clk)
		begin
			if rising_edge(clk) then
				if (rst = '1') or (burst_en_i = '0') then
					state <= RESET;
					
				else
					state <= next_state;
					
				end if;
			end if;
		end process;

	end block;
	------------------------------------------------------------------------


	------------------------------------------------------------------------
		
	burst_counter : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') or (bcount_en = '0') then
				bcount 		 <= (others => '0');
				bcount_done <= '0';
												
			else
				-- default
				bcount_done <= '0';
			
				if (bcount = reg_burst(15 downto 0)) then
					bcount_done <= '1';
					
				elsif (trig_sync_i = '1') then
					bcount <= bcount + '1';
					
				end if;		
			end if;
		end if;
	end process;
	---------------------------------------------------------------------------
	
	
  
	---------------------------------------------------------------------------
	long_timer_proc : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') or (long_timer_en = '0') then
			
				count_msec 	 <= (others => '0');
				count_msec_ro   <= '0';
				
				long_timer 		 <= (others => '0');
				long_timer_done <= '0';
								
			else

				-- Milli-second Scaler
				----------------------------------------
				-- default
				count_msec_ro <= '0';
			
				if (count_msec = conv_std_logic_vector(40000, 16)) then -- 1 ms
					count_msec_ro <= '1';
					count_msec		 <= (others => '0');
				
				else
					count_msec <= count_msec + '1';
					
				end if;		
				----------------------------------------
				
				
				-- Milli-seconds Counter
				----------------------------------------
				-- default
				long_timer_done <= '0';
				
				if (long_timer = long_timer_val) then
					long_timer_done <= '1';
					long_timer 		 <= (others => '0');
					
				elsif (count_msec_ro = '1') then
					long_timer <= long_timer + '1';
				
				end if;
				----------------------------------------
			end if;
		end if;
	end process;
	---------------------------------------------------------------------------
	
	

	---------------------------------------------------------------------------		
	-- Reg Control 
	---------------------------------------------------------------------------
	
		-- Write
	-------------------------------------------------------
	busy_control_register : process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				reg_control_wr <= (others => '0');
			
			elsif (db_wr = '1') and (ren_control_i = '1') then
				reg_control_wr <= db;
			
			end if;
		end if;
	end process;
	
	busy_force_reg <= reg_control_wr(1);
	
	clr_busy_trig_reg	<= not(reg_control_wr(0)) 	and ren_control_i and db_wr;
	
   clr_busy_burst    <= not(reg_control_wr(4))	and ren_control_i and db_wr;
	clr_busy_bto 		<= not(reg_control_wr(5))	and ren_control_i and db_wr;
   clr_busy_ibto 		<= not(reg_control_wr(6)) 	and ren_control_i and db_wr;
	--clr_busy_bburst   <= not(reg_control_wr(7)) 	and ren_control_i and db_wr;
	

	-- Read
	--------------------------------------------------------------
	reg_control_rd(0) 			 <= busy_trig_i;
	reg_control_rd(3 downto 1)  <= reg_control_wr(3 downto 1);
	reg_control_rd(4) 			 <= busy_burst;
	reg_control_rd(5) 			 <= busy_bto;
	reg_control_rd(6) 			 <= busy_ibto;
	--reg_control_rd(7) 			 <= busy_bburst;
	reg_control_rd(31 downto 7) <= reg_control_wr(31 downto 7);
	--reg_control_rd(31 downto 8) <= reg_control_wr(31 downto 8);
	
	db <= reg_control_rd when ((ren_control_i and db_rd) = '1') else (others => 'Z');
	
	

  
end rtl;
