-- hds header_start
--
-- VHDL Entity back_end_fpga.i2c_lm82_engine.symbol
--
-- Created:
--          by - phmpk.UNKNOWN (VMWARE-TEST)
--          at - 09:41:31 09/30/05
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

ENTITY i2c_lm82_engine IS
   PORT( 
      ce_i2c          : IN     std_logic;
      ce_i2c_rcv      : IN     std_logic;
      data_to_lm82    : IN     std_logic_vector (7 DOWNTO 0);
      i2c_abort       : IN     std_logic;
      i2c_init        : IN     std_logic;
      i2c_rd_nwr      : IN     std_logic;
      i2c_rst         : IN     std_logic;
      iclk40          : IN     std_logic;
      lm82_cmnd       : IN     std_logic_vector (7 DOWNTO 0);
      scl_source      : IN     std_logic;
      sda_in          : IN     std_logic;
      sr_out          : IN     std_logic;
      dir_sda         : OUT    std_logic;
      en_sr_in        : OUT    std_logic;
      en_sr_out       : OUT    std_logic;
      i2c_busy        : OUT    std_logic;
      i2c_err         : OUT    std_logic;
      ld_sr_out       : OUT    std_logic;
      scl_en          : OUT    std_logic;
      sda_out         : OUT    std_logic;
      sel_pointer     : OUT    std_logic;
      set_lm82_status : OUT    std_logic
   );

-- Declarations

END i2c_lm82_engine ;

-- hds interface_end
--
-- VHDL Architecture back_end_fpga.i2c_lm82_engine.fsm
--
-- Created:
--          by - phmpk.UNKNOWN (VMWARE-TEST)
--          at - 09:41:31 09/30/05
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

ARCHITECTURE fsm OF i2c_lm82_engine IS

   -- Architecture Declarations
   signal icount: natural range 0 to 9;
   signal pass_two: std_logic;
   signal abort: std_logic;

   TYPE STATE_TYPE IS (
      init,
      ld_sr_pointer,
      done,
      ld_sr_data,
      i2c_start,
      snd_addr,
      rcv_ack1,
      snd_data,
      rcv_ack2,
      i2c_stop1,
      rcv_data,
      snd_nack,
      error,
      sr_out1,
      sr_out2,
      sr_in,
      start_clk,
      i2c_stop_scl,
      i2c_stop2,
      ld_sr_data1,
      ld_sr_data2
   );

   -- State vector declaration
   ATTRIBUTE state_vector : string;
   ATTRIBUTE state_vector OF fsm : ARCHITECTURE IS "current_state" ;


   -- Declare current and next state signals
   SIGNAL current_state : STATE_TYPE ;
   SIGNAL next_state : STATE_TYPE ;

   -- Declare any pre-registered internal signals
   SIGNAL dir_sda_int : std_logic  ;
   SIGNAL en_sr_in_int : std_logic  ;
   SIGNAL en_sr_out_int : std_logic  ;
   SIGNAL i2c_busy_int : std_logic  ;
   SIGNAL i2c_err_int : std_logic  ;
   SIGNAL ld_sr_out_int : std_logic  ;
   SIGNAL scl_en_int : std_logic  ;
   SIGNAL sda_out_int : std_logic  ;
   SIGNAL set_lm82_status_int : std_logic  ;

BEGIN

   ----------------------------------------------------------------------------
   clocked : PROCESS(
      iclk40,
      i2c_rst
   )
   ----------------------------------------------------------------------------
   BEGIN
      IF (i2c_rst = '1') THEN
         current_state <= init;
         -- Reset Values
         dir_sda <= '1';
         en_sr_in <= '0';
         en_sr_out <= '0';
         i2c_busy <= '0';
         i2c_err <= '0';
         ld_sr_out <= '0';
         scl_en <= '0';
         sda_out <= '1';
         set_lm82_status <= '0';
         abort <= '0';
         icount <= 0;
         pass_two <= '0';
      ELSIF (iclk40'EVENT AND iclk40 = '1') THEN
         current_state <= next_state;
         -- Registered output assignments
         dir_sda <= dir_sda_int;
         en_sr_in <= en_sr_in_int;
         en_sr_out <= en_sr_out_int;
         i2c_busy <= i2c_busy_int;
         i2c_err <= i2c_err_int;
         ld_sr_out <= ld_sr_out_int;
         scl_en <= scl_en_int;
         sda_out <= sda_out_int;
         set_lm82_status <= set_lm82_status_int;

         -- Default Assignment To Internals
         abort <= i2c_abort or abort;
         icount <= 0;
         pass_two <= pass_two;

         -- Combined Actions for internal signals only
         CASE current_state IS
         WHEN init =>
            pass_two <= '0';
            abort <= '0';
         WHEN ld_sr_data =>
            pass_two <= '1';
         WHEN i2c_start =>
            icount <= 0;
         WHEN snd_addr =>
            icount <= icount;
         WHEN rcv_ack1 =>
            icount <= 0;
         WHEN snd_data =>
            icount <= icount;
         WHEN rcv_data =>
            icount <= icount;
         WHEN error =>
            abort <= '1';
         WHEN sr_out1 =>
            icount <= icount + 1;
         WHEN sr_out2 =>
            icount <= icount + 1;
         WHEN sr_in =>
            icount <= icount + 1;
         WHEN ld_sr_data1 =>
            pass_two <= '1';
         WHEN OTHERS =>
            NULL;
         END CASE;

      END IF;

   END PROCESS clocked;

   ----------------------------------------------------------------------------
   nextstate : PROCESS (
      abort,
      ce_i2c,
      ce_i2c_rcv,
      current_state,
      i2c_init,
      i2c_rd_nwr,
      icount,
      pass_two,
      scl_source,
      sda_in
   )
   ----------------------------------------------------------------------------
   BEGIN
      CASE current_state IS
      WHEN init =>
         IF (i2c_init = '1') THEN
            next_state <= ld_sr_pointer;
         ELSE
            next_state <= init;
         END IF;
      WHEN ld_sr_pointer =>
         IF (ce_i2c = '1') THEN
            next_state <= i2c_start;
         ELSE
            next_state <= ld_sr_pointer;
         END IF;
      WHEN done =>
            next_state <= init;
      WHEN ld_sr_data =>
         IF (ce_i2c = '1') THEN
            next_state <= i2c_start;
         ELSE
            next_state <= ld_sr_data;
         END IF;
      WHEN i2c_start =>
         IF (scl_source = '1') THEN
            next_state <= start_clk;
         ELSE
            next_state <= i2c_start;
         END IF;
      WHEN snd_addr =>
         IF (ce_i2c = '1') THEN
            next_state <= sr_out1;
         ELSE
            next_state <= snd_addr;
         END IF;
      WHEN rcv_ack1 =>
         IF ((ce_i2c_rcv = '1') and 
         (sda_in /= '0')) THEN
            next_state <= error;
         ELSIF ((ce_i2c = '1') AND
         ( i2c_rd_nwr = '1') AND
         (pass_two ='1')) THEN
            next_state <= rcv_data;
         ELSIF ((ce_i2c = '1')) THEN
            next_state <= snd_data;
         ELSE
            next_state <= rcv_ack1;
         END IF;
      WHEN snd_data =>
         IF ((ce_i2c = '1') AND
         pass_two = '0' AND
         ( i2c_rd_nwr = '0') AND 
         icount = 7) THEN
            next_state <= ld_sr_data1;
         ELSIF ((ce_i2c = '1') AND
         ( i2c_rd_nwr = '0') AND
         ( pass_two = '1') and 
         (icount = 7)) THEN
            next_state <= rcv_ack2;
         ELSIF ((ce_i2c = '1') AND
         ( i2c_rd_nwr = '1') AND
         (icount = 7)) THEN
            next_state <= rcv_ack2;
         ELSIF (ce_i2c = '1') THEN
            next_state <= sr_out2;
         ELSE
            next_state <= snd_data;
         END IF;
      WHEN rcv_ack2 =>
         IF ((ce_i2c_rcv = '1') and 
         (sda_in /= '0')) THEN
            next_state <= error;
         ELSIF (ce_i2c = '1') THEN
            next_state <= i2c_stop1;
         ELSE
            next_state <= rcv_ack2;
         END IF;
      WHEN i2c_stop1 =>
         IF (scl_source = '1') THEN
            next_state <= i2c_stop_scl;
         ELSE
            next_state <= i2c_stop1;
         END IF;
      WHEN rcv_data =>
         IF ((ce_i2c = '1') and 
         (icount = 8)) THEN
            next_state <= snd_nack;
         ELSIF (ce_i2c_rcv = '1') THEN
            next_state <= sr_in;
         ELSE
            next_state <= rcv_data;
         END IF;
      WHEN snd_nack =>
         IF (ce_i2c = '1') THEN
            next_state <= i2c_stop1;
         ELSE
            next_state <= snd_nack;
         END IF;
      WHEN error =>
         IF (ce_i2c = '1') THEN
            next_state <= i2c_stop1;
         ELSE
            next_state <= error;
         END IF;
      WHEN sr_out1 =>
         IF (icount = 7) THEN
            next_state <= rcv_ack1;
         ELSE
            next_state <= snd_addr;
         END IF;
      WHEN sr_out2 =>
            next_state <= snd_data;
      WHEN sr_in =>
            next_state <= rcv_data;
      WHEN start_clk =>
         IF (ce_i2c = '1') THEN
            next_state <= snd_addr;
         ELSE
            next_state <= start_clk;
         END IF;
      WHEN i2c_stop_scl =>
         IF (ce_i2c = '1') THEN
            next_state <= i2c_stop2;
         ELSE
            next_state <= i2c_stop_scl;
         END IF;
      WHEN i2c_stop2 =>
         IF ((ce_i2c = '1') AND ((pass_two = '1') OR
         ( i2c_rd_nwr = '0') OR 
         (abort = '1'))) THEN
            next_state <= done;
         ELSIF ((ce_i2c = '1') AND (i2c_rd_nwr = '1')) THEN
            next_state <= ld_sr_data;
         ELSE
            next_state <= i2c_stop2;
         END IF;
      WHEN ld_sr_data1 =>
            next_state <= ld_sr_data2;
      WHEN ld_sr_data2 =>
            next_state <= rcv_ack1;
      WHEN OTHERS =>
         next_state <= init;
      END CASE;

   END PROCESS nextstate;

   ----------------------------------------------------------------------------
   output : PROCESS (
      current_state,
      sr_out
   )
   ----------------------------------------------------------------------------
   BEGIN
      -- Default Assignment
      dir_sda_int <= '0';
      en_sr_in_int <= '0';
      en_sr_out_int <= '0';
      i2c_busy_int <= '1';
      i2c_err_int <= '0';
      ld_sr_out_int <= '0';
      scl_en_int <= '0';
      sda_out_int <= '1';
      set_lm82_status_int <= '0';
      -- Default Assignment To Internals

      -- Combined Actions
      CASE current_state IS
      WHEN init =>
         i2c_busy_int <= '0';
         dir_sda_int <= '1';
         i2c_err_int <= '0';
      WHEN ld_sr_pointer =>
         ld_sr_out_int <= '1';
      WHEN done =>
         set_lm82_status_int <= '1';
      WHEN ld_sr_data =>
         ld_sr_out_int <= '1';
      WHEN i2c_start =>
         dir_sda_int <= '0';
         sda_out_int <= '0';
      WHEN snd_addr =>
         dir_sda_int <= '0';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      WHEN rcv_ack1 =>
         dir_sda_int <= '1';
         scl_en_int <= '1';
         sda_out_int <= sr_out;
      WHEN snd_data =>
         dir_sda_int <= '0';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      WHEN rcv_ack2 =>
         dir_sda_int <= '1';
         scl_en_int <= '1';
         sda_out_int <='0';
      WHEN i2c_stop1 =>
         sda_out_int <= '0';
         scl_en_int <= '1';
         -- i2c_err_int <= i2c_err_int;
      WHEN rcv_data =>
         dir_sda_int <= '1';
         scl_en_int <= '1';
      WHEN snd_nack =>
         dir_sda_int <= '0';
         sda_out_int <= '1';
         scl_en_int <= '1';
      WHEN error =>
         dir_sda_int <= '1';
         i2c_err_int <= '1';
         scl_en_int <= '1';
         sda_out_int <='0';
      WHEN sr_out1 =>
         dir_sda_int <= '0';
         en_sr_out_int <= '1';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      WHEN sr_out2 =>
         dir_sda_int <= '0';
         en_sr_out_int <= '1';
         sda_out_int <= sr_out;
         scl_en_int <= '1';
      WHEN sr_in =>
         dir_sda_int <= '1';
         en_sr_in_int <= '1';
         scl_en_int <= '1';
      WHEN start_clk =>
         dir_sda_int <= '0';
         sda_out_int <= '0';
         scl_en_int <= '1';
      WHEN i2c_stop_scl =>
         sda_out_int <= '0';
         -- i2c_err_int <= i2c_err_int;
      WHEN i2c_stop2 =>
         sda_out_int <= '1';
         -- i2c_err_int <= i2c_err_int;
      WHEN ld_sr_data1 =>
         scl_en_int <= '1';
      WHEN ld_sr_data2 =>
         ld_sr_out_int <= '1';
         scl_en_int <= '1';
      WHEN OTHERS =>
         NULL;
      END CASE;

   END PROCESS output;

   -- Concurrent Statements
sel_pointer <= not pass_two;

END fsm;
